@@ -8,7 +8,7 @@
 //
 // dType.h: Basic template classes for elementary types
 //
-// Copyright (C) 2012	Xiuwen Zheng
+// Copyright (C) 2013	Xiuwen Zheng
 //
 // This file is part of CoreArray.
 //
@@ -29,7 +29,7 @@
  *	\file     dType.h
  *	\author   Xiuwen Zheng
  *	\version  1.0
- *	\date     2007 - 2012
+ *	\date     2007 - 2013
  *	\brief    Basic template classes for elementary types
  *	\details
 **/
@@ -45,7 +45,7 @@
 #endif
 
 #ifdef COREARRAY_UNIX
-#  include <sys/types.h>
+#  include <unistd.h>
 #endif
 
 #ifdef COREARRAY_MSC
@@ -55,27 +55,17 @@
 #endif
 
 #include <cfloat>
+#include <limits>
 #include <string>
 
 
 namespace CoreArray
 {
-	// CPU MACRO definition
-
-	#if (SIZE_MAX == UINT16_MAX) || (SIZE_MAX == INT16_MAX)
-	#  define COREARRAY_CPU 16
-	#elif (SIZE_MAX == UINT32_MAX) || (SIZE_MAX == INT32_MAX)
-	#  define COREARRAY_CPU 32
-	#elif (SIZE_MAX == UINT64_MAX) || (SIZE_MAX == INT64_MAX)
-	#  define COREARRAY_CPU 64
-	#else
-	#  error "Unable to determine CPU."
-	#endif
-
 	/// Memory data type id
 	enum TSVType {
 		svCustom = 0,   ///< Unknown or customized type
-		svCustomInt,    ///< Customized integer
+		svCustomInt,    ///< Customized signed integer
+		svCustomUInt,   ///< Customized unsigned integer
 		svCustomFloat,  ///< Customized float number
 		svCustomStr,    ///< Customized string type
 		svInt8,         ///< Signed integer of 8 bits
@@ -89,22 +79,29 @@ namespace CoreArray
 		svFloat32,      ///< Float number of single precision (32 bits)
 		svFloat64,      ///< Float number of double precision (64 bits)
 		svStrUTF8,      ///< UTF-8 string
-		svStrUTF16,     ///< UTF-16 string
-		svStrUTF32	///< UTF-32 string
+		svStrUTF16      ///< UTF-16 string
 	};
 
 
 	/// Whether x (TSVType) is an integer or not
-	#define COREARRAY_SVINT(x) \
-		((svInt8<=(x) && (x)<=svUInt64) || (x)==svCustomInt)
+	#define COREARRAY_SV_INTEGER(x) \
+		((svInt8<=(x) && (x)<=svUInt64) || (x)==svCustomInt || (x)==svCustomUInt)
+
+	/// Whether x (TSVType) is a signed integer or not
+	#define COREARRAY_SV_SINT(x) \
+		((x)==svInt8 || (x)==svInt16 || (x)==svInt32 || (x)==svInt64 || (x)==svCustomInt)
+
+	/// Whether x (TSVType) is an unsigned integer or not
+	#define COREARRAY_SV_UINT(x) \
+		((x)==svUInt8 || (x)==svUInt16 || (x)==svUInt32 || (x)==svUInt64 || (x)==svCustomUInt)
 
 	/// Whether x (TSVType) is a float number or not
-	#define COREARRAY_SVFLOAT(x) \
+	#define COREARRAY_SV_FLOAT(x) \
 		((x)==svFloat32 || (x)==svFloat64 || (x)==svCustomFloat)
 
 	/// Whether x (TSVType) is a string or not
-	#define COREARRAY_SVSTR(x) \
-		((x)==svStrUTF8 || (x)==svStrUTF16)
+	#define COREARRAY_SV_STRING(x) \
+		((x)==svStrUTF8 || (x)==svStrUTF16 || (x)==svCustomStr)
 
 
 	#define COREARRAY_TR_UNKNOWN    -1
@@ -257,8 +254,8 @@ namespace CoreArray
 		static const char * TraitName() { return "Int64"; }
 		static const char * StreamName() { return "dInt64"; }
 
-		COREARRAY_INLINE static Int64 Min() { return INT64_MIN; }
-		COREARRAY_INLINE static Int64 Max() { return INT64_MAX; }
+		COREARRAY_INLINE static Int64 Min() { return std::numeric_limits<Int64>::min(); }
+		COREARRAY_INLINE static Int64 Max() { return std::numeric_limits<Int64>::max(); }
 	};
 
 	template<> struct TdTraits<UInt64>
@@ -273,7 +270,7 @@ namespace CoreArray
 		static const char * StreamName() { return "dUInt64"; }
 
 		COREARRAY_INLINE static UInt64 Min() { return 0; }
-		COREARRAY_INLINE static UInt64 Max() { return UINT64_MAX; }
+		COREARRAY_INLINE static UInt64 Max() { return std::numeric_limits<UInt64>::max(); }
 	};
 
 
@@ -327,7 +324,7 @@ namespace CoreArray
 			typedef UInt32 Type;
 			typedef UInt64 TypeEx;
 			static const Type Val = UINT32_MAX;
-			static const TypeEx ValEx = UINT64_MAX;
+			static const TypeEx ValEx = UInt64(0) - UInt64(1); // UINT64_MAX;
 		};
 		template<> struct _Bit2Int<3, true>
 		{
@@ -431,7 +428,7 @@ namespace CoreArray
 		static const char * TraitName() { return StreamName()+1; }
 
 		COREARRAY_INLINE static Int64 Min()
-			{ return (bits > 0) ? 0 : ((-1ll) << (BITS<bits>::NumBit-1)); }
+			{ return (bits > 0) ? 0 : (Int64(-1) << (BITS<bits>::NumBit-1)); }
 		COREARRAY_INLINE static Int64 Max()
 			{ return (bits > 0) ? BITS<bits>::Mask : (BITS<bits>::Mask ^ BITS<bits>::Flag); }
 	};
@@ -519,26 +516,19 @@ namespace CoreArray
 	typedef char UTF8;
 
 #if (WCHAR_MAX == UINT16_MAX) || (WCHAR_MAX == INT16_MAX)
-#    define COREARRAY_SIZEOF_WCHAR 2
+#  define COREARRAY_SIZEOF_WCHAR 2
 	/// UTF-16 character
 	typedef wchar_t UTF16;
 	/// UTF-32 character
 	typedef Int32 UTF32;
 
 #elif (WCHAR_MAX == UINT32_MAX) || (WCHAR_MAX == INT32_MAX)
-#    define COREARRAY_SIZEOF_WCHAR 4
+#  define COREARRAY_SIZEOF_WCHAR 4
 	/// UTF-16 character
 	typedef Int16 UTF16;
 	/// UTF-32 character
 	typedef wchar_t UTF32;
 
-#elif (WCHAR_MAX == UINT64_MAX) || (WCHAR_MAX == INT64_MAX)
-#    define COREARRAY_SIZEOF_WCHAR 8
-	/// UTF-16 character
-	typedef Int16 UTF16;
-	/// UTF-32 character
-	typedef Int32 UTF32;
-
 #else
 #  error "Unable to determine sizeof(wchar_t)."
 #endif
